%Program to convert multiple ADCP power spectra to STWAVE formatted input
close all;
clear all;

[infilename inpathname]=uigetfile('Dspec*.*', 'Select ADCP Directional Wave Power Spectrum');

UL = 0.25;
LL = 0.04;
StwaveGridOrientation = 255;
GridRotation = StwaveGridOrientation-180;
MagneticDeclination = 0;

DR = dir([inpathname infilename(1:5) '*.*']);

tmptime = infilename(6:13);
if(str2num(tmptime(1:2))>=50)
    datetime = ['19' tmptime];
else
    datetime = ['20' tmptime];
end
filename = ['STW' datetime '.ADC'];

[outfilename outpathname] = uiputfile('*.*','Save ADCP/STWAVE Input file as:',filename);
DST = [-85:5:85];
fidlog = fopen([outpathname 'AdcpToStwaveLog.txt'],'wt');

for jj = 1:length(DR)
   clear infilename
   infilename = DR(jj).name
   [Sm,F,D,f,d,P,timestr,SpecTime]=ADCPP(infilename,inpathname,UL,LL);
   %P are the paramters:[Hmo,Tpeak,Dpeak,Tavg,Davg]
   %S = ((((((Sm./4)).^2))./(1000^2)))./(90/360);  %convert to m^2/Hz/deg
   %inclued in ADCPP now
   S = Sm;
   Fadcp = sum(S,2)./length(d);
   Fasum = 0;
   for i = 1:length(f)-1
       Fasum = Fasum + Fadcp(i)*(f(i+1)-f(i));
   end
   Fasum = Fasum + Fadcp(end)*(f(end)-f(end-1));
   Hmo = 4*sqrt(Fasum);
   dmag = d+MagneticDeclination;
   dt1 = (GridRotation-dmag).*pi/180;
   dt2 = atan2(sin(dt1),cos(dt1)).*180/pi;
   [dst,ii] = sort(dt2);
   S1 = S(:,ii);
   D1 = D(ii);
   S2 = zeros(length(f),length(DST));
   for i = 1:length(f)
       S2(i,:) = interp1(dst,S1(i,:),DST);
   end

   uli = find(f>=UL);
   ul = uli(1);

   lli = find(f<=LL);
   ll = lli(end);

   STW1 = S2(ll:ul,:); %this is m^2/Hz/degree
   STW = STW1;
   fst = f(ll:ul);
   Fa = sum(STW,2)./(length(dst));
   [MaxE,mi] = max(Fa);
   FreqPeak = fst(mi);
   Fd = sum(STW,1)./(length(fst));
   Fasum = 0;
   for i = 1:length(fst)-1
       Fasum = Fasum + Fa(i)*(fst(i+1)-fst(i));
   end
   Fasum = Fasum + Fa(end)*(fst(end)-fst(end-1));
   Hstw = 4*sqrt(Fasum);


   %write STWAVE spectrum to output file
   %write output file header
   tmptime = infilename(6:13);
   if(str2num(tmptime(1:2))>=50)
       datetime = ['19' tmptime];
   else
       datetime = ['20' tmptime];
   end
   WindSpeed = 0;
   StWindDir = 0;
   TideLevel = 0;

   filename = ['STW' datetime '.ADC'];

   fidout = fopen([outpathname filename],'wt');

   fprintf(fidout,'%g %g\n',length(fst),length(DST));
   fprintf(fidout,'%7.5f  ',fst);       
   fprintf(fidout,' \n');
   fprintf(fidout,'%s  ',datetime);
   fprintf(fidout,'%g  ',WindSpeed);
   fprintf(fidout,'%g  ',StWindDir);
   fprintf(fidout,'%g  ',FreqPeak);
   fprintf(fidout,'%g  ',TideLevel);
   fprintf(fidout,' \n');
   STWsum = 0;
   ainc = 5*pi/180;
   STWpeakcheck = 0;
   for i = 1:length(fst)
      if(i<length(fst))
          df = fst(i+1)-fst(i);
      else
          df = (fst(i)-fst(i-1));
      end
      for j = 1:length(DST)
%            fprintf(fidout,'%11.4e ',(STW(i,j)/(2*pi))*(P(1)^2/Hmo^2));  %convert to m^2/Hz/rad and normalize with wave height squared
%            STWsum = STWsum + ((STW(i,j)/(2*pi))*(P(1)^2/Hmo^2))*df*5*pi/180;
            fprintf(fidout,'%11.4e ',(STW(i,j)*180/pi));  %convert to m^2/Hz/rad no normalizing
            STWsum = STWsum + ((STW(i,j)))*df*ainc*180/pi;
            if(STW(i,j)>STWpeakcheck)
                STWpeakcheck = STW(i,j);
                STWJ = j;
                STWI = i;
            end
      end
      fprintf(fidout,'\n');
   end
   HMOstw = 4*sqrt(STWsum);
   
   %peak direction using polyfit across peak dir and two adjacent bins
   if(STWJ>1 & STWJ<length(DST))
      rxp = [DST(STWJ-1) DST(STWJ) DST(STWJ+1)];
      ryp = [STW(STWI,STWJ-1) STW(STWI,STWJ) STW(STWI,STWJ+1)];
      [coefs,tmpp]=polyfit(rxp,ryp,2);
      rxpp = [DST(STWJ-1):0.1:DST(STWJ+1)];
      rypp = polyval(coefs,rxpp);
      [maxep,iipoly]=max(rypp);
      Dpeakfstw = GridRotation-rxpp(iipoly);
      clear rxp ryp coefs tmpp rxpp rypp maxep iipoly;
   elseif(STWJ==1)
      Dpeakfstw = GridRotation-DST(1);
   elseif(STWJ==length(DST));
      Dpeakfstw = GridRotation-DST(end);
   end
  
   

   fprintf(fidlog,'%s %s %s %s %5.2f %5.2f %5.2f %5.2f %5.1f %5.1f\n',datetime(1:4),datetime(5:6),datetime(7:8),datetime(9:10),...
       HMOstw,P(1),1/FreqPeak,P(2),Dpeakfstw,P(3));
   fclose(fidout);
   clear Sm F D f d P timestr SpecTime
   clear S STW STW1 S1 S2 D1 Hmo Hstw
   clear dst fst tmptime datetime
   clear uli ul lli ll
end
fclose(fidlog);


break

figure;
  subplot(2,2,1)
  contour(d,f,S,100)
  title([datestr(SpecTime,0)]);
  grid on
  ylabel('Hz')
  axis([-180 180 0 UL]);
  set(gca,'xtick',[-180 -135 -90 -45 0 45 90 135 180])
  ax = axis;
  tx1 = ax(1)+((ax(2)-ax(1))/16);
  ty1 = 2*(ax(4)-ax(3))/16;
  text(tx1,ty1,['Hmo = ' num2str(Hmo,'%5.2f') ' m']);
  axis square

  subplot(2,2,3)
  plot(d,D,'k')
  hold
  plot([P(3) P(3)],[0 max(D)],'r')
  plot([P(5) P(5)],[0 max(D)],'b')
  ylabel('m^2/deg')
  templabel = {'Direction waves come from (deg)'};
  xlabel(templabel)
  grid on
  axis([-180 180 0 max(D)]);
  set(gca,'xtick',[-180 -135 -90 -45 0 45 90 135 180])
  ax = axis;
  tx1 = ax(1)+((ax(2)-ax(1))/16);
  ty1 = 15*(ax(4)-ax(3))/16;
  dy1 = 2*(ax(4)-ax(3))/16;
  h1 = text(tx1,ty1,['Dpk = ' num2str(round(P(3)),'%4.1f') ' deg']);
  set(h1,'Color','red');
  h1 = text(tx1,ty1-dy1,['Dmn = ' num2str(round(P(5)),'%4.1f') ' deg']);
  set(h1,'Color','blue');
  axis square
  
  subplot(2,2,2)
  plot(F,f,'k')
  hold
  plot([0 max(F)],[1/P(2) 1/P(2)],'r')
  plot([0 max(F)],[1/P(4) 1/P(4)],'b')
  xlabel('m^2/Hz')
  ylabel('Hz')
  grid on
  axis([0 max(F) 0 UL]);
  ax = axis;
  tx1 = ax(1)+8*((ax(2)-ax(1))/16);
  ty1 = 15*(ax(4)-ax(3))/16;
  dy1 = 2*(ax(4)-ax(3))/16;
  h1 = text(tx1,ty1,['Tpk = ' num2str(P(2),'%3.1f') ' s']);
  set(h1,'Color','red');
  h1 = text(tx1,ty1-dy1,['Tmn = ' num2str(P(4),'%3.1f') ' s']);
  set(h1,'Color','blue');
  axis square
  
  i = find(f>=UL);
  ii = i(1);
  subplot(2,2,4)
  mesh(d,f(1:ii),S(1:ii,1:end))
  axis([-180 180 0 UL 0 max(max(S))]);
  zlabel('m^2/Hz/deg')
  set(gca,'xtick',[-180 -90 0 90 180])

  set(gcf,'PaperOrientation','landscape');
  set(gcf,'PaperPosition',[.25 .25 10.5 8]);
%  jpgname = ['FourPanelPlot.jpg'];
%  print(gcf,'-djpeg90',jpgname);


%%%%STWAVE SPECTRUM
figure;
  subplot(2,2,1)
  contour(DST,fst,STW,100)
  title([datestr(SpecTime,0)]);
  grid on
  ylabel('Hz')
  axis([-180 180 0 UL]);
  set(gca,'xtick',[-180 -135 -90 -45 0 45 90 135 180])
  ax = axis;
  tx1 = ax(1)+((ax(2)-ax(1))/16);
  ty1 = 2*(ax(4)-ax(3))/16;
  text(tx1,ty1,['Hmo = ' num2str(Hstw,'%5.2f') ' m']);
  axis square

  subplot(2,2,3)
  plot(DST,Fd,'k')
  hold
  plot([GridRotation-P(3) GridRotation-P(3)],[0 max(Fd)],'r')
  plot([GridRotation-P(5) GridRotation-P(5)],[0 max(Fd)],'b')
  ylabel('m^2/deg')
  templabel = {'STWAVE, Direction waves go to (deg)'};
  xlabel(templabel)
  grid on
  axis([-180 180 0 max(Fd)]);
  set(gca,'xtick',[-180 -135 -90 -45 0 45 90 135 180])
  ax = axis;
  tx1 = ax(1)+((ax(2)-ax(1))/16);
  ty1 = 15*(ax(4)-ax(3))/16;
  dy1 = 2*(ax(4)-ax(3))/16;
  h1 = text(tx1,ty1,['Dpk = ' num2str(round(GridRotation-P(3)),'%4.1f') ' deg']);
  set(h1,'Color','red');
  h1 = text(tx1,ty1-dy1,['Dmn = ' num2str(round(GridRotation-P(5)),'%4.1f') ' deg']);
  set(h1,'Color','blue');
  axis square
  
  subplot(2,2,2)
  plot(Fa,fst,'k')
  hold
  plot([0 max(Fa)],[1/P(2) 1/P(2)],'r')
  plot([0 max(Fa)],[1/P(4) 1/P(4)],'b')
  xlabel('m^2/Hz')
  ylabel('Hz')
  grid on
  axis([0 max(Fa) 0 UL]);
  ax = axis;
  tx1 = ax(1)+8*((ax(2)-ax(1))/16);
  ty1 = 15*(ax(4)-ax(3))/16;
  dy1 = 2*(ax(4)-ax(3))/16;
  h1 = text(tx1,ty1,['Tpk = ' num2str(P(2),'%3.1f') ' s']);
  set(h1,'Color','red');
  h1 = text(tx1,ty1-dy1,['Tmn = ' num2str(P(4),'%3.1f') ' s']);
  set(h1,'Color','blue');
  axis square
  
  i = find(f>=UL);
  ii = i(1);
  subplot(2,2,4)
  mesh(DST,fst,STW)
  axis([-180 180 0 UL 0 max(max(STW))]);
  zlabel('m^2/Hz/deg')
  set(gca,'xtick',[-180 -90 0 90 180])

  set(gcf,'PaperOrientation','landscape');
  set(gcf,'PaperPosition',[.25 .25 10.5 8]);
%  jpgname = ['FourPanelPlot.jpg'];
%  print(gcf,'-djpeg90',jpgname);

